@@ -14,9 +14,14 @@ from .data_classes import (
     UnscheduledServiceEquipmentData,
     FixedCosts,
     FromDictMixin,
+    EquipmentMap,
+    StrategyMap,
+    PortConfig,
 )
+from .mixins import RepairsMixin
 from .environment import WombatEnvironment
 from .post_processor import Metrics
 from .repair_management import RepairManager
+from .port import Port
 from .service_equipment import ServiceEquipment
 from .simulation_api import Configuration, Simulation
