@@ -110,7 +110,17 @@ class WombatEnvironment(simpy.Environment):
             until = self.max_run_time
         elif until > self.max_run_time:
             until = self.max_run_time
-        super().run(until=until)
+        try:
+            super().run(until=until)
+        except BaseException as e:
+            # Flush the logs to so the simulation up to the point of failure is logged
+            self._events_logger.handlers[0].flush()
+            self._operations_logger.handlers[0].flush()
+            raise e
+
+        # Ensure all logged events make it to their target file
+        self._events_logger.handlers[0].flush()
+        self._operations_logger.handlers[0].flush()
 
     def _logging_setup(self) -> None:
         """Completes the setup for logging data."""
@@ -120,30 +130,25 @@ class WombatEnvironment(simpy.Environment):
             simulation = self.simulation_name.replace(" ", "_")
         dt_stamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S.%f")
 
-        self.events_log_fname = f"{dt_stamp}_{simulation}_events.log"
-        self.operations_log_fname = f"{dt_stamp}_{simulation}_operations.log"
-        self.power_potential_fname = f"{dt_stamp}_{simulation}_power_potential.csv"
-        self.power_production_fname = f"{dt_stamp}_{simulation}_power_production.csv"
+        events_log_fname = f"{dt_stamp}_{simulation}_events.log"
+        operations_log_fname = f"{dt_stamp}_{simulation}_operations.log"
+        power_potential_fname = f"{dt_stamp}_{simulation}_power_potential.csv"
+        power_production_fname = f"{dt_stamp}_{simulation}_power_production.csv"
+        metrics_input_fname = f"{dt_stamp}_{simulation}_metrics_inputs.yaml"
 
-        self.events_log_fname = str(
-            self.data_dir / "outputs" / "logs" / self.events_log_fname
-        )
-        self.operations_log_fname = str(
-            self.data_dir / "outputs" / "logs" / self.operations_log_fname
-        )
-        self.power_potential_fname = str(
-            self.data_dir / "outputs" / "logs" / self.power_potential_fname
-        )
-        self.power_production_fname = str(
-            self.data_dir / "outputs" / "logs" / self.power_production_fname
-        )
+        log_path = self.data_dir / "outputs" / "logs"
+        self.events_log_fname = log_path / events_log_fname
+        self.operations_log_fname = log_path / operations_log_fname
+        self.power_potential_fname = log_path / power_potential_fname
+        self.power_production_fname = log_path / power_production_fname
+        self.metrics_input_fname = log_path / metrics_input_fname
 
         _dir = self.data_dir / "outputs" / "logs"
         if not _dir.is_dir():
             _dir.mkdir()
 
-        setup_logger("events_log", self.events_log_fname)
-        setup_logger("operations_log", self.operations_log_fname)
+        setup_logger("events_log", self.events_log_fname, capacity=7000)
+        setup_logger("operations_log", self.operations_log_fname, capacity=7000)
         self._events_logger = logging.getLogger("events_log")
         self._operations_logger = logging.getLogger("operations_log")
 
@@ -152,9 +157,14 @@ class WombatEnvironment(simpy.Environment):
         """Current time within the simulation ("datetime" column within weather)."""
         now = self.now
         minutes = now % 1 * 60
-        dt = self.weather.iloc[math.floor(now)].name.to_pydatetime()
+        if now == self.max_run_time:
+            _dt = self.weather.iloc[math.floor(now - 1)].name.to_pydatetime()
+            _dt + timedelta(hours=1)
+        else:
+            _dt = self.weather.iloc[math.floor(now)].name.to_pydatetime()
+
         minutes, seconds = math.floor(minutes), math.ceil(minutes % 1 * 60)
-        return dt + timedelta(minutes=minutes, seconds=seconds)
+        return _dt + timedelta(minutes=minutes, seconds=seconds)
 
     def is_workshift(self, workday_start: int = -1, workday_end: int = -1) -> bool:
         """Checks if the current simulation time is within the windfarm's working hours.
@@ -398,6 +408,7 @@ class WombatEnvironment(simpy.Environment):
         part_ol: float | int = 0,
         duration: float = 0,
         request_id: str = "na",
+        location: str = "na",
         materials_cost: Union[int, float] = 0,
         hourly_labor_cost: Union[int, float] = 0,
         salary_labor_cost: Union[int, float] = 0,
@@ -430,6 +441,9 @@ class WombatEnvironment(simpy.Environment):
             an empty string for n/a, by default 0.
         request_id : str
             The ``RepairManager`` assigned request_id found in ``RepairRequest.request_id``, by default "na".
+        location : str
+            The location of where the event ocurred: should be one of site, port,
+            enroute, or system, by default "na".
         duration : float
             Length of time the action lasted, by default 0.
         materials_cost : Union[int, float], optional
@@ -441,6 +455,11 @@ class WombatEnvironment(simpy.Environment):
         equipment_cost : Union[int, float], optional
             Total cost of equipment for action, in USD, by default 0.
         """
+        valid_locations = ("site", "system", "port", "enroute", "na")
+        if location not in valid_locations:
+            raise ValueError(
+                f"Event logging `location` must be one of: {valid_locations}"
+            )
         self._events_logger.info(
             format_events_log_message(
                 self.simulation_time,
@@ -457,6 +476,7 @@ class WombatEnvironment(simpy.Environment):
                 additional,
                 duration,
                 request_id,
+                location,
                 materials_cost,
                 hourly_labor_cost,
                 salary_labor_cost,
@@ -491,6 +511,7 @@ class WombatEnvironment(simpy.Environment):
             "additional",
             "duration",
             "request_id",
+            "location",
             "materials_cost",
             "hourly_labor_cost",
             "salary_labor_cost",
@@ -565,15 +586,15 @@ class WombatEnvironment(simpy.Environment):
         events = self._create_events_log_dataframe().sort_values("env_time")
         operations = self._create_operations_log_dataframe().sort_values("env_time")
 
-        events_fname = self.events_log_fname.replace(".log", ".csv")
+        events_fname = self.events_log_fname.with_suffix(".csv")
         events.to_csv(events_fname, index=False)
 
-        operations_fname = self.operations_log_fname.replace(".log", ".csv")
+        operations_fname = self.operations_log_fname.with_suffix(".csv")
         operations.to_csv(operations_fname, index=False)
 
         if delete_original:
-            Path(self.operations_log_fname).unlink()
-            Path(self.events_log_fname).unlink()
+            self.operations_log_fname.unlink()
+            self.events_log_fname.unlink()
 
         if return_df:
             return operations, events
@@ -644,26 +665,43 @@ class WombatEnvironment(simpy.Environment):
             logging files are all deleted, by default False
         """
 
-        Path(self.events_log_fname).unlink()
-        Path(self.operations_log_fname).unlink()
+        # NOTE: Everything is wrapped in a try/except clause to protect against failure
+        # when inevitably a file has already been deleted on accident, or if in the
+        # dataframe generation step, the original logs were deleted
+
+        try:
+            self.events_log_fname.unlink()
+        except FileNotFoundError:
+            pass
+
+        try:
+            self.operations_log_fname.unlink()
+        except FileNotFoundError:
+            pass
+
         if not log_only:
             # Don't fail if any of the following files were not created
             try:
-                Path(self.events_log_fname.replace(".log", ".csv")).unlink()
+                self.events_log_fname.with_suffix(".csv").unlink()
             except FileNotFoundError:
                 pass
 
             try:
-                Path(self.operations_log_fname.replace(".log", ".csv")).unlink()
+                self.operations_log_fname.with_suffix(".csv").unlink()
             except FileNotFoundError:
                 pass
 
             try:
-                Path(self.power_potential_fname).unlink()
+                self.power_potential_fname.unlink()
             except FileNotFoundError:
                 pass
 
             try:
-                Path(self.power_production_fname).unlink()
+                self.power_production_fname.unlink()
+            except FileNotFoundError:
+                pass
+
+            try:
+                self.metrics_input_fname.unlink()
             except FileNotFoundError:
                 pass
