@@ -1,8 +1,10 @@
 """Provides a consistent way to access the provided Dinwoodie and IEA Task 26 data libraries."""
 
+from __future__ import annotations
+
 import os
 import re
-from typing import Any, Union
+from typing import Any
 from pathlib import Path
 
 import yaml  # type: ignore
@@ -10,10 +12,23 @@ import yaml  # type: ignore
 
 ROOT = Path(__file__).parents[2].resolve()
 DEFAULT_LIBRARY = ROOT / "library"
-DINWOODIE = DEFAULT_LIBRARY / "dinwoodie"
-IEA_26 = DEFAULT_LIBRARY / "iea26"
+CODE_COMPARISON = DEFAULT_LIBRARY / "code_comparison"
+BASE_CASES = DEFAULT_LIBRARY / "baseline"
 
-library_map = {"DINWOODIE": DINWOODIE, "IEA_26": IEA_26, "IEA26": IEA_26}
+DINWOODIE = CODE_COMPARISON / "dinwoodie"
+IEA_26 = DEFAULT_LIBRARY / "code_comparison" / "iea26"
+
+OSW_FIXED = BASE_CASES / "offshore_fixed"
+LBW = BASE_CASES / "land_based"
+
+library_map = {
+    "DINWOODIE": DINWOODIE,
+    "IEA_26": IEA_26,
+    "IEA26": IEA_26,
+    "OSW_FIXED": OSW_FIXED,
+    "LBW": LBW,
+    "LAND_BASED": LBW,
+}
 
 # YAML loader that is able to read scientific notation
 custom_loader = yaml.SafeLoader
@@ -33,14 +48,14 @@ custom_loader.add_implicit_resolver(
 )
 
 
-def load_yaml(path: Union[str, Path], fname: str) -> Any:
+def load_yaml(path: str | Path, fname: str | Path) -> Any:
     """Loads and returns the contents of the YAML file.
 
     Parameters
     ----------
-    path : Union[str, Path]
+    path : str | Path
         Path to the file to be loaded.
-    fname : str
+    fname : str | Path
         Name of the file (ending in .yaml) to be loaded.
 
     Returns
