@@ -1,7 +1,7 @@
 """The main API for the ``wombat``."""
 from __future__ import annotations
 
-from typing import List, Union, Optional
+from typing import Optional
 from pathlib import Path
 
 import yaml
@@ -17,22 +17,23 @@ from wombat.core import (
     WombatEnvironment,
 )
 from wombat.windfarm import Windfarm
+from wombat.core.port import Port
 from wombat.core.library import load_yaml, library_map
 
 
-def _library_mapper(file_path: Union[str, Path]) -> Path:
+def _library_mapper(file_path: str | Path) -> Path:
     """Attempts to extract a default library path if one of "DINWOODIE" or "IEA_26"
     are passed, other returns ``file_path``.
 
     Parameters
     ----------
-    file_path : Union[str, Path]
+    file_path : str | Path
         Should be a valid file path, or one of "DINWOODIE" or "IEA_26" to indicate a
         provided library is being used.
 
     Returns
     -------
-    Union[str, Path]
+    str | Path
         The library path.
     """
     return Path(library_map.get(file_path, file_path)).resolve()  # type: ignore
@@ -50,7 +51,7 @@ class Configuration(FromDictMixin):
         The data directory. See ``wombat.simulation.WombatEnvironment`` for more details.
     layout : str
         The windfarm layout file. See ``wombat.Windfarm`` for more details.
-    service_equipment : Union[str, List[str]]
+    service_equipment : str | list[str]
         The equpiment that will be used in the simulation. See
         ``wombat.core.ServiceEquipment`` for more details.
     weather : str
@@ -66,8 +67,11 @@ class Configuration(FromDictMixin):
         The annual inflation rate to be used for post-processing.
     fixed_costs : str
         The file name for the fixed costs assumptions.
-    project_capacity : Union[int, float]
+    project_capacity : int | float
         The total capacity of the wind plant, in MW.
+    port : dict | str | Path
+        The port configuration file or dictionary that will be used to setup a
+        tow-to-port repair strategy, default None.
     start_year : int
         Start year of the simulation. The exact date will be determined by
         the first valid date of this year in ``weather``.
@@ -82,13 +86,14 @@ class Configuration(FromDictMixin):
     name: str
     library: Path = field(converter=_library_mapper)
     layout: str
-    service_equipment: Union[str, List[str]]
-    weather: Union[str, pd.DataFrame]
+    service_equipment: str | list[str]
+    weather: str | pd.DataFrame
     workday_start: int
     workday_end: int
     inflation_rate: float
     fixed_costs: str
-    project_capacity: Union[int, float]
+    project_capacity: int | float
+    port: dict | str | Path = field(default=None)
     start_year: int = field(default=None)
     end_year: int = field(default=None)
     SAM_settings: str = field(default=None)
@@ -122,13 +127,14 @@ class Simulation(FromDictMixin):
     env: WombatEnvironment = field(init=False)
     repair_manager: RepairManager = field(init=False)
     service_equipment: list[ServiceEquipment] = field(init=False)
+    port: Port = field(init=False)
 
     def __attrs_post_init__(self) -> None:
         self._setup_simulation()
 
     @config.validator  # type: ignore
     def _create_configuration(
-        self, attribute: Attribute, value: Union[str, dict, Configuration]
+        self, attribute: Attribute, value: str | dict | Configuration
     ) -> None:
         """Validates the configuration object and creates the ``Configuration`` object
         for the simulation.Raises:
@@ -165,14 +171,14 @@ class Simulation(FromDictMixin):
             )
 
     @classmethod
-    def from_config(cls, config: Union[str, dict, Configuration]):
+    def from_config(cls, config: str | dict | Configuration):
         """Creates the ``Simulation`` object only the configuration contents as either a
         full file path to the configuration file, a dictionary of the configuration
         contents, or pre-loaded ``Configuration`` object.
 
         Parameters
         ----------
-        config : Union[str, dict, Configuration]
+        config : str | dict | Configuration
             The simulation configuration, see ``Configuration`` for more details on the
             contents. The following is a description of the acceptable contents:
 
@@ -225,6 +231,11 @@ class Simulation(FromDictMixin):
                     self.env, self.windfarm, self.repair_manager, service_equipment
                 )
             )
+        if self.config.port is not None:
+            self.port = Port(
+                self.env, self.windfarm, self.repair_manager, self.config.port
+            )
+            self.service_equipment.extend(self.port.tugboat_manager.items)
 
     def run(
         self,
@@ -238,7 +249,7 @@ class Simulation(FromDictMixin):
 
         Parameters
         ----------
-        until : Optional[Union[int, float, Event]], optional
+        until : Optional[int | float | Event], optional
             When to stop the simulation, by default None. See documentation on
             ``simpy.Environment.run`` for more details.
         create_metrics : bool, optional
@@ -274,7 +285,7 @@ class Simulation(FromDictMixin):
             fixed_costs=self.config.fixed_costs,
             substation_id=self.windfarm.substation_id.tolist(),
             turbine_id=self.windfarm.turbine_id.tolist(),
-            service_equipment_names=[el.settings.name for el in self.service_equipment],
+            service_equipment_names=[el.settings.name for el in self.service_equipment],  # type: ignore
             SAM_settings=self.config.SAM_settings,
         )
 
@@ -284,10 +295,10 @@ class Simulation(FromDictMixin):
         """
         data = dict(
             data_dir=str(self.config.library),
-            events=self.env.events_log_fname.replace(".log", ".csv"),
-            operations=self.env.operations_log_fname.replace(".log", ".csv"),
-            potential=self.env.power_potential_fname,
-            production=self.env.power_production_fname,
+            events=str(self.env.events_log_fname.with_suffix(".csv")),
+            operations=str(self.env.operations_log_fname.with_suffix(".csv")),
+            potential=str(self.env.power_potential_fname),
+            production=str(self.env.power_production_fname),
             inflation_rate=self.config.inflation_rate,
             project_capacity=self.config.project_capacity,
             turbine_capacities=[
@@ -296,18 +307,9 @@ class Simulation(FromDictMixin):
             fixed_costs=self.config.fixed_costs,
             substation_id=self.windfarm.substation_id.tolist(),
             turbine_id=self.windfarm.turbine_id.tolist(),
-            service_equipment_names=[el.settings.name for el in self.service_equipment],
+            service_equipment_names=[el.settings.name for el in self.service_equipment],  # type: ignore
             SAM_settings=self.config.SAM_settings,
         )
 
-        # Get the filename and rename it apporpriately
-        events_fname = Path(self.env.events_log_fname)
-        fname = events_fname.name
-        fname = fname.replace("_events.log", "_metrics_inputs.yaml")
-
-        # Get the path and move it 1 level up to <data_dir>/outputs/
-        fpath = events_fname.parents[1]
-
-        fname = fpath / fname  # type: ignore
-        with open(fname, "w") as f:
+        with open(self.env.metrics_input_fname, "w") as f:
             yaml.dump(data, f, default_flow_style=False, sort_keys=False)
