@@ -12,14 +12,7 @@ from geopy import distance  # type: ignore
 from wombat.core import RepairManager, WombatEnvironment
 from wombat.core.library import load_yaml
 from wombat.windfarm.system import Cable, System
-
-
-try:  # pylint: disable=duplicate-code
-    from functools import cache  # type: ignore
-except ImportError:  # pylint: disable=duplicate-code
-    from functools import lru_cache  # pylint: disable=duplicate-code
-
-    cache = lru_cache(None)  # pylint: disable=duplicate-code
+from wombat.utilities.utilities import cache
 
 
 class Windfarm:
@@ -114,6 +107,15 @@ class Windfarm:
         self.graph = windfarm
 
     def _create_turbines_and_substations(self) -> None:
+        """Instantiates the turbine and substation models as defined in the
+        user-provided layout file, and connects these models to the appropriate graph
+        nodes to create a fully representative windfarm network model.
+
+        Raises
+        ------
+        ValueError
+            Raised if the subassembly data is not provided in the layout file.
+        """
         for system_id, data in self.graph.nodes(data=True):
             if data["subassembly"] == "":
                 raise ValueError(
@@ -133,6 +135,15 @@ class Windfarm:
             )
 
     def _create_cables(self) -> None:
+        """Instantiates the cable models as defined in the user-provided layout file,
+        and connects these models to the appropriate graph edges to create a fully
+        representative windfarm network model.
+
+        Raises
+        ------
+        ValueError
+            Raised if the cable model is not specified.
+        """
         for start_node, end_node, data in self.graph.edges(data=True):
             # Check that the cable data is provided
             if data["cable"] == "":
@@ -170,7 +181,7 @@ class Windfarm:
             data["latitude"], data["longitude"] = end_points.mean(axis=0)
 
     def calculate_distance_matrix(self) -> None:
-        """Calculates hte geodesic distance, in km, between all of the windfarm's nodes, e.g.,
+        """Calculates the geodesic distance, in km, between all of the windfarm's nodes, e.g.,
         substations and turbines, and cables.
         """
         ids = list(self.graph.nodes())
